unit MultipleClients3U;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TMainForm = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormDockDrop(Sender: TObject; Source: TDragDockObject; X,
      Y: Integer);
  protected
    function CreateDockManager: IDockManager; override;
  end;

  TCustomDockManager = class(TDockTree)
  protected
    procedure PaintDockFrame(Canvas: TCanvas; Control: TControl;
      const ARect: TRect); override;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

{ TCustomDockManager }

type
  TControlAccess = class(TControl);

procedure TCustomDockManager.PaintDockFrame(Canvas: TCanvas;
  Control: TControl; const ARect: TRect);
const
  GrabberSize = 12; //Hardcoded in Controls unit

  procedure DrawCloseButton(Left, Top: Integer);
  begin
    DrawFrameControl(Canvas.Handle, Rect(Left, Top, Left+GrabberSize-2,
      Top+GrabberSize-2), DFC_CAPTION, DFCS_CAPTIONCLOSE);
  end;

  procedure DrawCaptionBar(Left, Top, Right, Bottom: Integer);
  begin
    Canvas.Brush.Color := clActiveCaption;
    Canvas.FillRect(Rect(Left, Top, Right, Bottom));
  end;

  procedure DrawCaptionText(const Text: String; Left, Top, Right, Bottom: Integer);
  begin
    Canvas.Font.Name := 'Arial';
    Canvas.Font.Color := clCaptionText;
    Canvas.Font.Height := Succ(Top - Bottom);
    Canvas.TextRect(Rect(Left, Top, Right, Bottom), Left, Top, Text);
  end;

begin
  with ARect do
    if DockSite.Align in [alTop, alBottom] then
    begin
      DrawCaptionBar(Left, Top, Left+GrabberSize-1, Bottom);
      //Cannot draw vertical text simply, so don't bother
      DrawCloseButton(Left+1, Top+1);
    end
    else
    begin
      DrawCaptionBar(Left, Top, Right, Top+GrabberSize-1);
      DrawCaptionText(TControlAccess(Control).Caption,
        Left, Top, Right, Top+GrabberSize-2);
      DrawCloseButton(Right-GrabberSize+2, Top+1);
    end;
end;

{ TMainForm }

function TMainForm.CreateDockManager: IDockManager;
begin
  //If this is a dock site, we should use the dock manager,
  //and we do not already have one, then create one
  if (DockManager = nil) and DockSite and UseDockManager then
    Result := TCustomDockManager.Create(Self)
  else
    Result := DockManager;
  //Set double-buffered as original VCL code does
  DoubleBuffered := DoubleBuffered or Assigned(Result);
end;

procedure TMainForm.FormCreate(Sender: TObject);
const
  Colors: array[1..6] of TColor =
    (clWhite, clBlack, clBlue, clGreen, clRed, clYellow);
var
  I: Integer;
begin
  //DockManager := TCustomDockManager.Create(Self);
  for I := Low(Colors) to High(Colors) do
    with TForm.CreateNew(Self) do
    begin
      Caption := 'Dock me in the main form';
      Color := Colors[I];
      DragKind := dkDock;
      DragMode := dmAutomatic;
      Position := poDefaultPosOnly;
      Width := 230;
      Height := 100;
      Visible := True;
    end;
end;

procedure TMainForm.FormDockDrop(Sender: TObject; Source: TDragDockObject;
  X, Y: Integer);
begin
  DockManager.ResetBounds(True)
end;

end.
